@@ -5,8 +5,9 @@
 #include <winternl.h>
 #include <ntstatus.h>
 #include <windows.h>
+#include <psapi.h>
 
-const char* ffDetectSwap(FFlist* result)
+const char* detectByNqsi(FFlist* result)
 {
     uint8_t buffer[4096];
     ULONG size = sizeof(buffer);
@@ -21,11 +22,32 @@ const char* ffDetectSwap(FFlist* result)
         ffStrbufInitNWS(&swap->name, current->FileName.Length / sizeof(wchar_t), current->FileName.Buffer);
         if (ffStrbufStartsWithS(&swap->name, "\\??\\"))
             ffStrbufSubstrAfter(&swap->name, strlen("\\??\\") - 1);
-        swap->bytesUsed = current->TotalUsed * pageSize;
-        swap->bytesTotal = current->CurrentSize * pageSize;
+        swap->bytesUsed = (uint64_t) current->TotalUsed * pageSize;
+        swap->bytesTotal = (uint64_t) current->CurrentSize * pageSize;
         if (current->NextEntryOffset == 0)
             break;
     }
+    return NULL;
+}
+
+const char* detectByKgpi(FFlist* result)
+{
+    PERFORMANCE_INFORMATION pi = {};
+    if (!K32GetPerformanceInfo(&pi, sizeof(pi)))
+        return "K32GetPerformanceInfo(&pi, sizeof(pi)) failed";
+    FFSwapResult* swap = ffListAdd(result);
+    ffStrbufInitS(&swap->name, "Page File");
+    swap->bytesTotal = (uint64_t) (pi.CommitLimit > pi.PhysicalTotal ? pi.CommitLimit - pi.PhysicalTotal : 0) * pi.PageSize;
+    swap->bytesUsed = (uint64_t) (pi.CommitTotal > pi.PhysicalTotal ? pi.CommitTotal - pi.PhysicalTotal : 0) * pi.PageSize;
 
     return NULL;
 }
+
+const char* ffDetectSwap(FFlist* result)
+{
+    const char* err = detectByNqsi(result);
+    if (err == NULL)
+        return NULL;
+
+    return detectByKgpi(result);
+}
