@@ -2,10 +2,12 @@
 #include "common/io/io.h"
 #include "common/processing.h"
 #include "common/properties.h"
+#include "util/path.h"
 #include "util/stringUtils.h"
 #include "util/binary.h"
 
 #include <ctype.h>
+#include <stdint.h>
 #ifdef __FreeBSD__
     #include <paths.h>
     #ifndef _PATH_LOCALBASE
@@ -20,13 +22,16 @@
 #include "util/windows/version.h"
 #include <windows.h>
 
-static bool getFileVersion(const FFstrbuf* exePath, FFstrbuf* version)
+static bool getFileVersion(const FFstrbuf* exePath, const wchar_t* stringName, FFstrbuf* version)
 {
-    wchar_t exePathW[PATH_MAX];
+    wchar_t exePathW[PATH_MAX + 1];
     int len = MultiByteToWideChar(CP_UTF8, 0, exePath->chars, (int)exePath->length, exePathW, ARRAY_SIZE(exePathW));
     if (len <= 0) return false;
-    return ffGetFileVersion(exePathW, version);
+    assert(len < (int) ARRAY_SIZE(exePathW));
+    exePathW[len] = L'\0';
+    return ffGetFileVersion(exePathW, stringName, version);
 }
+
 #elif __HAIKU__
     #include "util/haiku/version.h"
 #endif
@@ -60,12 +65,10 @@ static bool extractBashVersion(const char* line, FF_MAYBE_UNUSED uint32_t len, v
     return false;
 }
 
-static bool getShellVersionBash(FFstrbuf* exe, FFstrbuf* exePath, FFstrbuf* version)
+static bool getShellVersionBash(FFstrbuf* exe, FFstrbuf* version)
 {
-    const char* path = exePath->chars;
-    if (*path == '\0')
-        path = exe->chars;
-    ffBinaryExtractStrings(path, extractBashVersion, version, (uint32_t) strlen("@(#)Bash version 0.0.0(0) release GNU"));
+    ffBinaryExtractStrings(exe->chars, extractBashVersion, version, (uint32_t) strlen("@(#)Bash version 0.0.0(0) release GNU"));
+    if (version->length > 0) return true;
 
     if(!getExeVersionRaw(exe, version))
         return false;
@@ -81,8 +84,11 @@ static bool getShellVersionFish(FFstrbuf* exe, FFstrbuf* version)
     if(!getExeVersionRaw(exe, version))
         return false;
 
-    //fish, version 3.6.0
-    ffStrbufSubstrAfterLastC(version, ' ');
+    //fish, version 4.0.2-1 (Built by MSYS2 project) // version can be localized if LC_ALL is set
+    if (version->length < strlen("fish, v") || !ffStrbufStartsWithS(version, "fish")) return false;
+    uint32_t index = ffStrbufNextIndexC(version, strlen("fish, "), ' ');
+    ffStrbufSubstrAfter(version, index);
+    ffStrbufSubstrBeforeFirstC(version, ' ');
     return true;
 }
 
@@ -98,7 +104,7 @@ static bool getShellVersionPwsh(FFstrbuf* exe, FFstrbuf* version)
     }
 
     #ifdef _WIN32
-    if(getFileVersion(exe, version))
+    if(getFileVersion(exe, NULL, version))
     {
         ffStrbufSubstrBeforeLastC(version, '.');
         return true;
@@ -114,35 +120,39 @@ static bool getShellVersionPwsh(FFstrbuf* exe, FFstrbuf* version)
 
 static bool getShellVersionKsh(FFstrbuf* exe, FFstrbuf* version)
 {
-#if __OpenBSD__ || __NetBSD__
+    if(ffProcessAppendStdErr(version, (char* const[]) {
+        exe->chars,
+        "--version",
+        NULL
+    }) == NULL && ffStrbufSubstrAfterFirstS(version, " (AT&T Research) "))
+    {
+        //  version         sh (AT&T Research) 93u+ 2012-08-01
+        ffStrbufSubstrBeforeFirstC(version, ' ');
+        return true;
+    }
+
+    ffStrbufClear(version);
     if(ffProcessAppendStdOut(version, (char* const[]) {
         exe->chars,
         "-c",
         "echo $KSH_VERSION",
         NULL
-    }) != NULL)
-        return false;
+    }) == NULL && ffStrbufSubstrAfterFirstS(version, " KSH "))
+    {
+        // OKSH: @(#)PD KSH v5.2.14 99/07/13.2
+        // MKSH: @(#)MIRBSD KSH R59 2025/04/26 +Debian
+        // $OKSH_VERSION doesn't exist on OpenBSD
+        ffStrbufSubstrBeforeFirstC(version, ' ');
+        ffStrbufTrimLeft(version, 'v');
+        return true;
+    }
 
-    // @(#)PD KSH v5.2.14 99/07/13.2
-    ffStrbufSubstrAfterFirstC(version, 'v');
-    ffStrbufSubstrBeforeFirstC(version, ' ');
-#else
-    if(ffProcessAppendStdErr(version, (char* const[]) {
-        exe->chars,
-        "--version",
-        NULL
-    }) != NULL)
-        return false;
-
-    //  version         sh (AT&T Research) 93u+ 2012-08-01
-    ffStrbufSubstrAfterLastC(version, ')');
-    ffStrbufTrim(version, ' ');
-#endif
-    return true;
+    return false;
 }
 
 static bool getShellVersionOksh(FFstrbuf* exe, FFstrbuf* version)
 {
+    // Homebrew version
     if(ffProcessAppendStdOut(version, (char* const[]) {
         exe->chars,
         "-c",
@@ -222,13 +232,9 @@ static bool extractZshVersion(const char* line, FF_MAYBE_UNUSED uint32_t len, vo
     return false;
 }
 
-static bool getShellVersionZsh(FFstrbuf* exe, FFstrbuf* exePath, FFstrbuf* version)
+static bool getShellVersionZsh(FFstrbuf* exe, FFstrbuf* version)
 {
-    const char* path = exePath->chars;
-    if (*path == '\0')
-        path = exe->chars;
-
-    ffBinaryExtractStrings(path, extractZshVersion, version, (uint32_t) strlen("zsh-0.0-0"));
+    ffBinaryExtractStrings(exe->chars, extractZshVersion, version, (uint32_t) strlen("zsh-0.0-0"));
     if (version->length) return true;
 
     return getExeVersionGeneral(exe, version); //zsh 5.9 (arm-apple-darwin21.3.0)
@@ -255,7 +261,7 @@ static bool getShellVersionWinPowerShell(FFstrbuf* exe, FFstrbuf* version)
 }
 #endif
 
-bool fftsGetShellVersion(FFstrbuf* exe, const char* exeName, FFstrbuf* exePath, FFstrbuf* version)
+bool fftsGetShellVersion(FFstrbuf* exe, const char* exeName, FFstrbuf* version)
 {
     if (!instance.config.general.detectVersion) return false;
 
@@ -263,9 +269,9 @@ bool fftsGetShellVersion(FFstrbuf* exe, const char* exeName, FFstrbuf* exePath,
         return false;
 
     if(ffStrEqualsIgnCase(exeName, "bash"))
-        return getShellVersionBash(exe, exePath, version);
+        return getShellVersionBash(exe, version);
     if(ffStrEqualsIgnCase(exeName, "zsh"))
-        return getShellVersionZsh(exe, exePath, version);
+        return getShellVersionZsh(exe, version);
     if(ffStrEqualsIgnCase(exeName, "fish"))
         return getShellVersionFish(exe, version);
     if(ffStrEqualsIgnCase(exeName, "pwsh"))
@@ -274,7 +280,7 @@ bool fftsGetShellVersion(FFstrbuf* exe, const char* exeName, FFstrbuf* exePath,
         return getExeVersionGeneral(exe, version); //tcsh 6.24.07 (Astron) 2022-12-21 (aarch64-apple-darwin) options wide,nls,dl,al,kan,sm,rh,color,filec
     if(ffStrEqualsIgnCase(exeName, "nu"))
         return getShellVersionNushell(exe, version);
-    if(ffStrEqualsIgnCase(exeName, "ksh"))
+    if(ffStrEqualsIgnCase(exeName, "ksh") || ffStrEqualsIgnCase(exeName, "mksh"))
         return getShellVersionKsh(exe, version);
     if(ffStrEqualsIgnCase(exeName, "oksh"))
         return getShellVersionOksh(exe, version);
@@ -286,12 +292,14 @@ bool fftsGetShellVersion(FFstrbuf* exe, const char* exeName, FFstrbuf* exePath,
         return getShellVersionAsh(exe, version);
     if(ffStrEqualsIgnCase(exeName, "xonsh"))
         return getShellVersionXonsh(exe, version);
+    if(ffStrEqualsIgnCase(exeName, "brush"))
+        return getExeVersionGeneral(exe, version); // brush 0.2.23 (git:2835487)
 
     #ifdef _WIN32
     if(ffStrEqualsIgnCase(exeName, "powershell") || ffStrEqualsIgnCase(exeName, "powershell_ise"))
         return getShellVersionWinPowerShell(exe, version);
 
-    return getFileVersion(exe, version);
+    return getFileVersion(exe, NULL, version);
     #endif
 
     return false;
@@ -315,7 +323,7 @@ static bool extractGeneralVersion(const char *str, FF_MAYBE_UNUSED uint32_t len,
 
 FF_MAYBE_UNUSED static bool getTerminalVersionGnome(FFstrbuf* exe, FFstrbuf* version)
 {
-    if (exe->chars[0] == '/')
+    if (ffIsAbsolutePath(exe->chars))
     {
         ffBinaryExtractStrings(exe->chars, extractGeneralVersion, version, (uint32_t) strlen("0.0.0"));
         if (version->length) return true;
@@ -335,7 +343,7 @@ FF_MAYBE_UNUSED static bool getTerminalVersionGnome(FFstrbuf* exe, FFstrbuf* ver
 
 FF_MAYBE_UNUSED static bool getTerminalVersionXfce4Terminal(FFstrbuf* exe, FFstrbuf* version)
 {
-    if (exe->chars[0] == '/')
+    if (ffIsAbsolutePath(exe->chars))
     {
         ffBinaryExtractStrings(exe->chars, extractGeneralVersion, version, (uint32_t) strlen("0.0.0"));
         if (version->length) return true;
@@ -572,13 +580,51 @@ static bool getTerminalVersionZed(FFstrbuf* exe, FFstrbuf* version)
     return true;
 }
 
+static bool extractSshdVersion(const char *str, FF_MAYBE_UNUSED uint32_t len, void *userdata)
+{
+    if (!ffStrStartsWith(str, "OpenSSH_") || !ffCharIsDigit(str[strlen("OpenSSH_")])) return true;
+    str += strlen("OpenSSH_");
+    int count = 0;
+    sscanf(str, "%*d.%*dp%*d%n", &count);
+    if (count == 0) return true;
+    ffStrbufSetS((FFstrbuf*) userdata, str);
+    return false;
+}
+
+static bool getTerminalVersionSshd(FFstrbuf* exe, FFstrbuf* version)
+{
+    FF_STRBUF_AUTO_DESTROY exePath = ffStrbufCreate();
+    if (ffIsAbsolutePath(exe->chars))
+        ffStrbufSet(&exePath, exe);
+    else if (ffFindExecutableInPath("sshd", &exePath) != NULL)
+        return false;
+
+    ffBinaryExtractStrings(exePath.chars, extractSshdVersion, version, (uint32_t) strlen("OpenSSH0.0"));
+    if (version->length) return true;
+
+    if(ffProcessAppendStdOut(version, (char* const[]) {
+        exePath.chars,
+        "-V",
+        NULL
+    }) != NULL)
+        return false;
+
+    if (ffStrbufStartsWithS(version, "unknown ")) // `unknown option -- V` (ancient OpenSSH version)
+        ffStrbufSubstrAfterFirstC(version, '\n');
+
+    // OpenSSH_10.0p2 Ubuntu-5ubuntu5, OpenSSL 3.5.3 16 Sep 2025
+    ffStrbufSubstrAfterFirstC(version, '_');
+    ffStrbufSubstrBeforeFirstC(version, ',');
+    return true;
+}
+
 #ifndef _WIN32
 static bool getTerminalVersionKitty(FFstrbuf* exe, FFstrbuf* version)
 {
-    #if defined(__linux__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__)
+    #if defined(__linux__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__GNU__)
     char buffer[1024] = {};
     if (
-        #ifdef __linux__
+        #if __linux__ || __GNU__
         ffReadFileData(FASTFETCH_TARGET_DIR_USR "/lib64/kitty/kitty/constants.py", ARRAY_SIZE(buffer) - 1, buffer) ||
         ffReadFileData(FASTFETCH_TARGET_DIR_USR "/lib/kitty/kitty/constants.py", ARRAY_SIZE(buffer) - 1, buffer)
         #else
@@ -671,7 +717,7 @@ FF_MAYBE_UNUSED static bool getTerminalVersionPtyxis(FF_MAYBE_UNUSED FFstrbuf* e
 
 FF_MAYBE_UNUSED static bool getTerminalVersionTilix(FFstrbuf* exe, FFstrbuf* version)
 {
-    if (exe->chars[0] == '/')
+    if (ffIsAbsolutePath(exe->chars))
     {
         ffBinaryExtractStrings(exe->chars, extractGeneralVersion, version, (uint32_t) strlen("0.0.0"));
         if (version->length) return true;
@@ -707,6 +753,20 @@ FF_MAYBE_UNUSED static bool getTerminalVersionSakura(FFstrbuf* exe, FFstrbuf* ve
     ffStrbufSubstrAfterLastC(version, ' ');
     return true;
 }
+
+FF_MAYBE_UNUSED static bool getTerminalVersionTermite(FFstrbuf* exe, FFstrbuf* version)
+{
+    if(ffProcessAppendStdOut(version, (char* const[]) {
+        exe->chars,
+        "--version",
+        NULL
+    }) != NULL) // termite v16.9\nvte 0.78.1 +BIDI +GNUTLS +ICU +SYSTEMD
+        return false;
+
+    ffStrbufSubstrBeforeFirstC(version, '\n');
+    ffStrbufSubstrAfterLastC(version, 'v');
+    return true;
+}
 #endif
 
 #ifdef _WIN32
@@ -723,7 +783,7 @@ static bool getTerminalVersionWindowsTerminal(FFstrbuf* exe, FFstrbuf* version)
         return true;
     }
 
-    return getFileVersion(exe, version);
+    return getFileVersion(exe, NULL, version);
 }
 
 static bool getTerminalVersionConEmu(FFstrbuf* exe, FFstrbuf* version)
@@ -733,7 +793,7 @@ static bool getTerminalVersionConEmu(FFstrbuf* exe, FFstrbuf* version)
     if(version->length)
         return true;
 
-    return getFileVersion(exe, version);
+    return getFileVersion(exe, NULL, version);
 }
 
 #endif
@@ -749,7 +809,7 @@ bool fftsGetTerminalVersion(FFstrbuf* processName, FF_MAYBE_UNUSED FFstrbuf* exe
 
     #endif
 
-    #if defined(__linux__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__sun) || defined(__NetBSD__) || defined(__HAIKU__)
+    #if defined(__linux__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__sun) || defined(__NetBSD__) || defined(__HAIKU__) || defined(__GNU__)
 
     if(ffStrbufStartsWithIgnCaseS(processName, "gnome-terminal"))
         return getTerminalVersionGnome(exe, version);
@@ -812,6 +872,9 @@ bool fftsGetTerminalVersion(FFstrbuf* processName, FF_MAYBE_UNUSED FFstrbuf* exe
     if(ffStrbufIgnCaseEqualS(processName, "sakura"))
         return getTerminalVersionSakura(exe, version);
 
+    if(ffStrbufIgnCaseEqualS(processName, "termite"))
+        return getTerminalVersionTermite(exe, version);
+
     #endif
 
     #ifdef _WIN32
@@ -822,6 +885,9 @@ bool fftsGetTerminalVersion(FFstrbuf* processName, FF_MAYBE_UNUSED FFstrbuf* exe
     if(ffStrbufStartsWithIgnCaseS(processName, "ConEmu"))
         return getTerminalVersionConEmu(exe, version);
 
+    if(ffStrbufIgnCaseEqualS(processName, "warp.exe"))
+        return getFileVersion(exe, L"ProductVersion", version);
+
     #endif
 
     #ifndef _WIN32
@@ -895,9 +961,12 @@ bool fftsGetTerminalVersion(FFstrbuf* processName, FF_MAYBE_UNUSED FFstrbuf* exe
     if(ffStrbufStartsWithIgnCaseS(processName, "tmux"))
         return getTerminalVersionTmux(exe, version);
 
+    if(ffStrbufIgnCaseEqualS(processName, "sshd") || ffStrbufStartsWithIgnCaseS(processName, "sshd-"))
+        return getTerminalVersionSshd(exe, version);
+
     #ifdef _WIN32
 
-    return getFileVersion(exe, version);
+    return getFileVersion(exe, NULL, version);
 
     #else
 
