@@ -1,13 +1,12 @@
 #pragma once
 
-// This file will be included in "fastfetch.h", do NOT put unnecessary things here
-
 #include "common/option.h"
 
 typedef struct FFSeparatorOptions
 {
-    FFModuleBaseInfo moduleInfo;
-
     FFstrbuf string;
     FFstrbuf outputColor;
+    uint32_t times;
 } FFSeparatorOptions;
+
+static_assert(sizeof(FFSeparatorOptions) <= FF_OPTION_MAX_SIZE, "FFSeparatorOptions size exceeds maximum allowed size");
